/*  ___  ___  ___ ___
** / __|/ _ \/ __/ __| secs
** \__ \  __/ (__\__ \ Copyright 1999 Daniel Reed <n@ml.org>
** |___/\___|\___|___/ Simple Embedded Client Scripting
*/
#include <secs.h>

void	secs_script_init(void) {
	secs_script_expand(NULL, NULL);
}

char	*secs_script_expand(secs_block_t *block, char *instr) {
	register char	*str = instr;
	static char	*workstr;
	static size_t	worklen;

	if (instr == NULL) {
		if (block == NULL) {
			workstr = NULL;
			worklen = 0;
		}
		return(NULL);
	}
	if (block == NULL)
		block = secs_block_getroot();
	if (workstr == NULL) {
		assert(worklen == 0);
		workstr = secs_mem_alloc(256);
		worklen = 256;
	} else {
		assert(worklen != 0);
		bzero(workstr, worklen);
	}
	while (*str) {
		size_t	slen = strlen(workstr);

		if ((worklen - slen) < 2) {
			worklen += 256;
			workstr = secs_mem_realloc(workstr, worklen);
		}
		switch (*str) {
			case '$': {
				secs_var_t	*var = NULL;
				char	*tmp = NULL;

				if ((var = secs_var_find_n(NULL, str+1))
					!= NULL) {
				  str += strlen(var->name);
				  tmp = var->val;
				  while (*tmp) {
					size_t	slen = strlen(workstr);

					if ((worklen - slen) < 2) {
						worklen += 256;
						workstr =
							secs_mem_realloc(
							workstr, worklen);
					}
					workstr[slen] = *tmp;
					tmp++;
				  }
				  break;
				}
			}
			default:
				workstr[slen] = *str;
		}
		str++;
	}
	if ((worklen - strlen(workstr)) > 256) {
		worklen -= 256;
		workstr = secs_mem_realloc(workstr, worklen);
	}
	return(workstr);
}

int	secs_script_parse(char *line) {
	char	*strbuf = NULL, *str = NULL;
	size_t	slen = 0;

	debug_printf("secs_parse(``%s'')\n", line);
	if ((line == NULL) || ((slen = strlen(line)) == 0))
		return(0);
	strbuf = secs_mem_alloc(slen+2);
	if (strbuf == NULL)
		return(0);
	strncpy(strbuf, line, slen+2);
	while ((str = strchr(strbuf, '\n')) != NULL)
		*str = ' ';
	while ((str = strrchr(strbuf, ';')) != NULL)
		*str = 0;
	str = strbuf;
	while ((str != NULL) && (*str != 0)) {
		register char	*workstr = str;
		char	*word = NULL;

		str += strlen(str)+1;	// firstwhite() munges str, so we
					// have to do this before processing
		if (*workstr == '/') {
			secs_handle(workstr+1);
			continue;
		}
		word = atom(workstr);
		workstr = firstwhite(workstr);
		if (strcasecmp(word, "ECHO") == 0) {
			printf("%s\n", secs_script_expand(NULL, workstr));
		} else if (strcasecmp(word, "SET") == 0) {
			word = atom(workstr);
			workstr = firstwhite(workstr);
			secs_setvar(word, workstr);
		} else
			debug_printf("secs_parse: ``%s'' is an invalid"
				" command name (%s).\n", word, workstr);
	}
	secs_mem_free(strbuf);
	return(1);
}
