/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
** 
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>

extern WINDOW	*win_input;
extern buddywin_t	*curbwin, nullbwin;
extern faimconf_t	faimconf;
extern int	iamaway;

void	conio_exit(char *params) {
	if (params != NULL) {
		printf("/EXIT and /QUIT take no paramaters.\n");
		return;
	}
	aim_logoff();
	wshutitdown();
	/* So we don't quite conform to the structured programming ideal.
	 * Oh well.  JMM */
	exit(0);
}

void	conio_info(char *SN) {
	if (SN == NULL) {
		if (curbwin == &nullbwin) {
			printf("You cannot get info on status.\n");
			return;
		}
		SN = curbwin->winname; /* SN's scope is shorter then curbwin's. */
	}
	aim_getinfo(SN);
}

void	conio_closewin(char *SN) {
	buddywin_t	*bwin, *bbefore;

	if (SN == NULL) {
		if ((bwin = curbwin) == &nullbwin) {
			printf("You cannot close the status window.\n");
			return;
		}
	} else
		if ((bwin = bgetwin(SN)) == NULL) {
			printf("No window is open for %s.\n", SN);
			return;
		}
//	printf("Closed buddy window %s.\n", bwin->winname);
	bbefore = bwin->next;
	while (bbefore->next != bwin)
		bbefore = bbefore->next;
	bbefore->next = bwin->next;
	if (bwin == curbwin)
		curbwin = bwin->next;
	bremove(bwin);
	bupdate();
	touchwin(curbwin->win);
}

void	conio_openwin(char *SN) {
	if (strlen(SN) > 10) {
		printf("Screen names must be 10 characters or shorter.\n");
	} else {
		if (bgetwin(SN) != NULL)
			printf("There is already a window open for %s.\n",
				SN);
		else {
			buddywin_t	*bwin = NULL;

			printf("Opened window for %s.\n", SN);
			bnewwin(SN);
			bwin = bgetwin(SN);
			assert(bwin != NULL);
			bwin->offline = 0;
			bwin->notbuddy = 1;
			wprintf(bwin->win, IMWIN_STATUS, 1,
				"Non-buddy window created.\n");
			bupdate();
		}
	}
}

void	conio_sendim(char *SN, char *msg) {
	time_t	t = 0;
	struct tm	*tmptr = NULL;

	assert(SN != NULL);
	assert(*SN != 0);
	t = time(NULL);
	assert(t != 0);
	tmptr = localtime(&t);
	assert(tmptr != NULL);
	if (strlen(SN) > 10) {
		printf("Screen names must be 10 characters or shorter.\n");
		return;
	}
	if (msg == NULL) {
		aim_getinfo(SN);
		return;
	}
	if (curbwin == bgetwin(SN))
		hwprintf(curbwin->win, IMWIN_TEXT_U,
			"[%02i:%02i:%02i] <B>%s</B>: %s<br>",
			tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec,
			aim_connection.sn, msg);
	else
		hwprintf(curbwin->win, IMWIN_TEXT_U,
			"[%02i:%02i:%02i] -> *<B>%s</B>* %s<br>",
			tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec, SN,
			msg);
	logim(aim_connection.sn, SN, msg);
	naim_send_im(SN, msg);
}

void	conio_jump(char *SN) {
	buddywin_t	*bwin = NULL;

	if (SN == NULL) {
		bwin = curbwin;
		while (((bwin = bwin->next) != curbwin) && (bwin->waiting == 0))
			;
	} else
		if ((bwin = bgetwin(SN)) == NULL) {
			printf("No such buddy %s.\n", SN);
			return;
		}
	if (bwin != curbwin) {
		curbwin = bwin;
		bupdate();
		touchwin(curbwin->win);
	}
}

void	conio_reload(char *file) {
	char	buf[256];

	killconf(&faimconf);
	if (file == NULL) {
		snprintf(buf, sizeof(buf), "%s/.faimrc", getenv("HOME"));
		file = buf;
	}
	printf("Re-reading %s\n", file);
	naim_read_config(file);
	wsetp();
	naim_login_phase3d_f(NULL);
}

void	conio_away(char *awaymsg) {
	if (awaymsg == NULL) {
		if (iamaway)
			unsetaway();
		else
			setaway();
		return;
	}
	secs_setvar("awaymsg", awaymsg);
	setaway();
}

int	conio_handlecmd(char *buf) {
	char	line[1024];
	char	*cmd, *parms;

	assert(buf != NULL);
	strncpy(line, buf, sizeof(line)-1);
	line[sizeof(line)-1] = 0;
	cmd = atom(line);
	parms = firstwhite(line);
	if (script_doalias(cmd, parms) == 1)
		return(1);
	if (strcasecmp(cmd, "EVAL") == 0) {
		secs_script_parse(parms);
	} else if ((strcasecmp(cmd, "IM") == 0)
		|| (strcasecmp(cmd, "M") == 0)
		|| (strcasecmp(cmd, "MSG") == 0)) {
		conio_sendim(atom(parms), firstwhite(parms));
	} else if (strcasecmp(cmd, "CLEAR") == 0) {
		wclear(curbwin->win);
	} else if (strcasecmp(cmd, "RELOAD") == 0) {
		conio_reload(parms);
	} else if (strcasecmp(cmd, "AWAY") == 0) {
		conio_away(parms);
	} else if (strcasecmp(cmd, "SET") == 0) {
		set_setvar(parms);
	} else if ((strcasecmp(cmd, "OPEN") == 0)
		|| (strcasecmp(cmd, "WINDOW") == 0)) {
		conio_openwin(parms);
	} else if ((strcasecmp(cmd, "CLOSE") == 0)
		|| (strcasecmp(cmd, "ENDWIN") == 0)) {
		conio_closewin(parms);
	} else if ((strcasecmp(cmd, "HELP") == 0)
		|| (strcasecmp(cmd, "ABOUT") == 0)) {
		help_printhelp(parms);
	} else if ((strcasecmp(cmd, "J") == 0)
		|| (strcasecmp(cmd, "JUMP") == 0)) {
		conio_jump(parms);
	} else if ((strcasecmp(cmd, "QUIT") == 0)
		|| (strcasecmp(cmd, "EXIT") == 0)) {
		conio_exit(parms);
	} else if (strcasecmp(cmd, "INFO") == 0) {
		conio_info(parms);
	} else
		return(0);
	return(1);
}
int      (*secs_client_cmdhandler)(char *) = conio_handlecmd;

void	conio_handleline(char *line) {
	if (curbwin == &nullbwin) {
		if (*line == '/') {
			if (conio_handlecmd(line+1) == 0)
				conio_openwin(line+1);
		} else
			conio_sendim(atom(line), firstwhite(line));
	} else {
		if (*line == '/')
			if (conio_handlecmd(line+1) != 0)
				return;
		curbwin->keepafterso = 1;
		conio_sendim(curbwin->winname, line);
	}
}

void	gotkey(int c) {
#ifndef NO_HISTORY
	HISTORY_STATE	*hstate = NULL;
	static int	histpos;
#endif
	static int	bufloc;
	static char	buf[1024], gotesc, inbold, initalic;

	updateidletime();
	if (c == 0) {
		bzero(buf, 1024);
		bufloc = initalic = inbold = gotesc = 0;
#ifndef NO_HISTORY
		histpos = 0;
#endif
		return;
	}
	assert(curbwin != NULL);
	if (c == 27) {
		if (!gotesc) {
			gotesc = 1;
			printf("Press Escape again to exit.\n");
		} else {
			printf("Logging off...\n");
			aim_logoff();
			wshutitdown();
			exit(0);
		}
		return;
	}
	gotesc = 0;
	switch(c) {
		case ERR:
			break;
		case KEY_UP:
#ifndef NO_HISTORY
			hstate = history_get_history_state();
			assert(hstate != NULL);
			if (hstate->entries == NULL)
				break;
			if (histpos > 0) {
				histpos--;
				assert(hstate->entries[histpos] != NULL);
				bzero(buf, 1024);
				strncpy(buf,
					(hstate->entries[histpos])->line, 1023);
				wclear(win_input);
				wprintf(win_input, INPUT_TEXT, 1, "%s", buf);
				bufloc = strlen(buf);
			}
#endif
			break;
		case KEY_DOWN:
#ifndef NO_HISTORY
			hstate = history_get_history_state();
			assert(hstate != NULL);
			if (hstate->entries == NULL)
				break;
			if (histpos < hstate->length) {
				histpos++;
				bzero(buf, 1024);
				if (hstate->entries[histpos] != NULL)
					strncpy(buf,
						(hstate->entries[histpos])->line, 1023);
				wclear(win_input);
				wprintf(win_input, INPUT_TEXT, 1, "%s", buf);
				bufloc = strlen(buf);
			}
#endif
			break;
		case KEY_LEFT:
			if (bufloc > 0) {
				waddstr(win_input, "\b");
				bufloc--;
			}
			break;
		case KEY_RIGHT:
			if (buf[bufloc] != 0) {
				wprintf(win_input, INPUT_TEXT, 1, "%c",
					buf[bufloc]);
				bufloc++;
			}
			break;
		case '\t':
			curbwin = curbwin->next;
			bupdate();
			touchwin(curbwin->win);
			break;
		case '\b': case KEY_BACKSPACE:
			if (bufloc < 1)
				break;
			buf[--bufloc] = 0;
			waddstr(win_input, "\b \b");
			break;
		case '\r': case '\n':
//			assert(((bufloc == 0) && (*buf == 0))
//				|| ((bufloc > 0) && (*buf != 0)));
			if (bufloc == 0)
				break;
#ifndef NO_HISTORY
			add_history(buf);
#endif
			conio_handleline(buf);
#ifndef NO_HISTORY
			hstate = history_get_history_state();
			assert(hstate != NULL);
			histpos = hstate->length;
#endif
			bzero(buf, 1024);
			inbold = initalic = bufloc = 0;
			wclear(win_input);
			break;
		case '<':
			buf[bufloc++] = '&';
			buf[bufloc++] = 'l';
			buf[bufloc++] = 't';
			buf[bufloc++] = ';';
			wprintf(win_input, INPUT_TEXT_ALT, 1, "&lt;");
			break;
		case '>':
			buf[bufloc++] = '&';
			buf[bufloc++] = 'g';
			buf[bufloc++] = 't';
			buf[bufloc++] = ';';
			wprintf(win_input, INPUT_TEXT_ALT, 1, "&gt;");
			break;
		case '&':
			buf[bufloc++] = '&';
			buf[bufloc++] = 'a';
			buf[bufloc++] = 'm';
			buf[bufloc++] = 'p';
			buf[bufloc++] = ';';
			wprintf(win_input, INPUT_TEXT_ALT, 1, "&amp;");
			break;
		case KEY_F(1): case 11:
			help_printhelp(NULL);
			break;
		case KEY_F(2): case 2:
			buf[bufloc++] = '<';
			if (inbold) {
				buf[bufloc++] = '/';
				wprintf(win_input, INPUT_TEXT_ALT, 1, "</B>");
			} else
				wprintf(win_input, INPUT_TEXT_ALT, 1, "<B>");
			buf[bufloc++] = 'B';
			buf[bufloc++] = '>';
			inbold = (inbold == 0);
			break;
		case KEY_F(3): case 22:
			buf[bufloc++] = '<';
			if (initalic) {
				buf[bufloc++] = '/';
				wprintf(win_input, INPUT_TEXT_ALT, 1,  "</I>");
			} else
				wprintf(win_input, INPUT_TEXT_ALT, 1, "<I>");
			buf[bufloc++] = 'I';
			buf[bufloc++] = '>';
			initalic = (initalic == 0);
			break;
		case KEY_F(4):
			printf("F4 is currently unused.\n");
			break;
		case KEY_F(5):
//			printf("F5 is currently unused.\n");
			buf[bufloc++] = '&';
			wprintf(win_input, INPUT_TEXT_ALT, 1, "&");
			break;
		case KEY_F(6): case 12:
			wclear(curbwin->win);
			break;
		case KEY_F(7): case 18: 
			conio_reload(NULL);
			break;
		case KEY_F(8): case 14:
			conio_jump(NULL);
			break;
		case KEY_F(9): case 1:
			if (iamaway)
				unsetaway();
			else
				setaway();
			break;
		case KEY_F(10): case 23: 
			conio_closewin(NULL);
			break;
		case KEY_F(11):
			buf[bufloc++] = '<';
			wprintf(win_input, INPUT_TEXT_ALT, 1, "<");
			break;
		case KEY_F(12):
			buf[bufloc++] = '>';
			wprintf(win_input, INPUT_TEXT_ALT, 1, ">");
			break;
		default:
			if (isprint(c)) {
				buf[bufloc++] = c;
				wprintf(win_input, INPUT_TEXT, 1, "%c", c);
				break;
			}
	}
}
